#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2015 IAS / CNRS / Univ. Paris-Sud
# BSD License - see attached LICENSE file
# Author: Alexandre Boucaud <alexandre.boucaud@ias.u-psud.fr>
r"""
 ____        ____  _   _ _____ ____
|  _ \ _   _|  _ \| | | | ____|  _ \
| |_) | | | | |_) | |_| |  _| | |_) |
|  __/| |_| |  __/|  _  | |___|  _ <
|_|    \__, |_|   |_| |_|_____|_| \_\
       |___/
---
Python-based PSF Homogenization kERnels
"""
